

## ------------------------------------------------------------------------ ##
##  R scripts for " A New Parametric Approach to Smoothing and Forecasting  ##
##                 of Mortality "                                           ##
## ------------------------------------------------------------------------ ##
##  Script        Scripts for Local likelihood smoothing -  Script n•03.1   ##
## ------------------------------------------------------------------------ ##
##  Description   Script to smooth the forces of mortality by local kernel  ##
##                weighted log-likelihood                                   ##
## ------------------------------------------------------------------------ ##
##  Author        Tomas Julien                                              ##
## ------------------------------------------------------------------------ ##
##  Version       01 - 19/10/2012                                           ##
## ------------------------------------------------------------------------ ##

## ---------------- If necessary,

rm(list=ls(all=TRUE))

## ------------------------------------------------------------------------ ##
##           	       Change the workspace repository                      ##
## ------------------------------------------------------------------------ ##

setwd("...")

## ------------------------------------------------------------------------ ##
##           	               Load functions                               ##
## ------------------------------------------------------------------------ ##

## --- Miscellaneous functions

source("Misc_functions.R")

## --- Graphic functions

require(lattice); require(grid); require(latticeExtra)

## --- Load X11 options

X11.options(type="nbcairo",family="FreeSans")

## --- IMPORTANT: package gam have to be launch before locfit else additive models will not work !!

require(gam)
require(locfit)
require(mgcv)

## ------------------------------------------------------------------------ ##
##           	                Load the data                               ##
## ------------------------------------------------------------------------ ##

load("D_h.RData")

load("ETR_h.RData")

load("TG05_m_h.RData")

load("INSEE_m_h.RData")

## ------------------------------------------------------------------------ ##
##           	                 The models                                 ##
## ------------------------------------------------------------------------ ##

## --- Smoothing by local kernel-weighted log-likelihood

## - mod.1: D_x ~ Poisson(ETR_x * exp(f(x))) Local likelihood with ETR as offset.
## - Mod.2.INSEE: D_x ~ Poisson(ETR_x * exp(f(log(mu_INSEE_x)))) Relational model including forces of mortality according to INSEE reference table.
## - Mod.2.TG05: D_x ~ Poisson(ETR_x * exp(f(log(mu_TGHF05_x))) Relational model including forces of mortality according to TGHF05 reference table.
## - Mod.3.INSEE: D_x ~ Poisson(ETR_x * mu_INSEE_x * exp(f(x))) Local likelihood including expected number of death according to INSEE table.
## - Mod.3.TG05: D_x ~ Poisson(ETR_x * mu_TGHF05_x * exp(f(x))) Local likelihood including expected number of death according to TGHF05 table.
## - Mod.4.INSEE: D_x ~ Poisson(ETR_x * exp(f_1(x) + f_2(log(mu_INSEE_x)))) Relational model including forces of mortality according to INSEE table.
## - Mod.4.TG05: D_x ~ Poisson(ETR_x * exp(f_1(x) + f_2(log(mu_TGHF05_x)))) Relational model including forces of mortality according to TGHF05 reference table.

## - For local likelihood we use the locfit package to speed up the computation and we only vary the bandwidth and the polynomial degree, keeping the weight function fixed (kernel = triweight)

fam <- "poisson"; lk <- "log"; k <- "triweight";

Age <- 30:90; Year <- 1996:2007
new.data <- expand.grid(age=Age, year=Year)

## ------------------------------------------------------------------------ ##
##           	                   mod.1                                    ##
## ------------------------------------------------------------------------ ##

## ---------------- Male population

y <- D_h
x1 <- rep(Age,length(Year))
x2 <- rep(Year,each= length(Age))
x3 <- ETR_h

## --- Optimal Constellation of Smoothing Parameters

hh <- 1 : 20; pp <- 0 : 3

## - Compute criteria and Fitted DF

m.crit.1_h <- get.crit.lk(y, Year, Age, hh, pp, k, fam, lk, c(x3))

## - Plot the criteria vs Fitted DF

x11();
plot.crit("AIC", m.crit.1_h$AICMAT, m.crit.1_h$V2MAT, c(1000,1600), c(3,100))
x11();
plot.crit("LCV", m.crit.1_h$LCVMAT, m.crit.1_h$V2MAT, c(1100,1600), c(3,100))

## - Keep h = 12 & p = 1 with LCV/AIC criterion

x11();
plot.crit("LCV", m.crit.1_h$LCVMAT, m.crit.1_h$V2MAT, c(1000,1600), c(3,100))
trellis.focus("panel", 1, 1, highlight = FALSE)
panel.points(m.crit.1_h$V2MAT[12,2], m.crit.1_h$LCVMAT[12,2], pch=16, col=gray(.91), cex=3, lwd = 2)
panel.points(m.crit.1_h$V2MAT[12,2], m.crit.1_h$LCVMAT[12,2], pch="1", col=1, cex=1.5)
trellis.unfocus()

## --- Compute the fit

## - Get the span and number of observations participating to the smoothing

nber.p <- get.enp.target(12); nber.p
span <- nber.p / (length(Age)*length(Year)); span

## - Fit the model

mod.1_h <- locfit(c(y) ~ lp(c(x1),c(x2),deg = 1, nn = span,  scale=1),ev=dat(), family = fam, link = lk, kern = k, weights=c(x3))

fit.m1_h <- matrix(predict(mod.1_h),length(Age),length(Year))
colnames(fit.m1_h) <- as.character(Year)

x11();
surface_plot(log(fit.m1_h), expression(paste("log ",widehat(mu)[h])), c(min(Age),max(Age), min(Year), max(Year)))

## --- Have a look on the single fit

## - Fit for a particular single year

year.single <- "1996"
x11();
plot(Age, (D_h/ETR_h)[, year.single], ylim=c(0,0.5))
points(Age, fit.m1_h[, year.single], type = "l", col = 2)

## - Fit for a particular single anc

age.single <- 90
x11();
plot(Year, (D_h/ETR_h)[age.single + 1 - min(Age), ])
points(Year, fit.m1_h[age.single + 1 - min(Age), ],type = "b", col = 2)

## - Transformed Residuals plot / Against age (2d)

x11();
surface_plot((D_h/ETR_h)-fit.m1_h, "Residuals",  c(min(Age),max(Age), min(Year), max(Year)))

## - Transformed Residuals plot / Against age (3.INSEE)
## - For a particular year. With smoothed line

x11();
year.single <-  "2000"
res <- (D_h/ETR_h)[,year.single] - fit.m1_h[,year.single]
fit.res <- locfit(res  ~ Age, family = "gaussian", alpha = 0.1, 
link = "identity", kern="tcub")
plot(Age, res, col=1, ylim = c(-0.2, 0.2), ylab = "Residuals", xlab = "Age")
points(Age, fitted(fit.res), col = 2, type = "l")

## ------------------------------------------------------------------------ ##
##           	                   mod.2.INSEE                              ##
## ------------------------------------------------------------------------ ##

## ---------------- Male population

y <- D_h
x1 <- INSEE_m_h[, as.character(Year)]
x2 <- ETR_h

## - Get the span and number of observations participating to the smoothing

nber.p <- get.enp.target(12); nber.p
span <- nber.p / (length(Age)*length(Year)); span

## - Fit the model

mod.2.INSEE_h <- locfit(c(y) ~ lp(c(log(x1)),deg = 1, nn = span,  scale=1),ev=dat(), family = fam, link = lk, kern = k, weights=c(x2))

fit.m2.INSEE_h <- matrix(predict(mod.2.INSEE_h),length(Age),length(Year))
colnames(fit.m2.INSEE_h) <- as.character(Year)

x11();
surface_plot(log(fit.m2.INSEE_h), expression(paste("log ",widehat(mu)[h])), c(min(Age),max(Age), min(Year), max(Year)))

## ------------------------------------------------------------------------ ##
##           	                   mod.2.TG05                               ##
## ------------------------------------------------------------------------ ##

## ---------------- Male population

y <- D_h
x1 <- TG05_m_h[,as.character(Year)]
x2 <- ETR_h

## - Get the span and number of observations participating to the smoothing

nber.p <- get.enp.target(12); nber.p
span <- nber.p / (length(Age)*length(Year)); span

## - Fit the model

mod.2.TG05_h <- locfit(c(y) ~ lp(c(log(x1)),deg = 1, nn = span,  scale=1),ev=dat(), family = fam, link = lk, kern = k, weights=c(x2))

fit.m2.TG05_h <- matrix(predict(mod.2.TG05_h),length(Age),length(Year))
colnames(fit.m2.TG05_h) <- as.character(Year)

x11();
surface_plot(log(fit.m2.TG05_h), expression(paste("log ",widehat(mu)[h])), c(min(Age),max(Age), min(Year), max(Year)))

## ------------------------------------------------------------------------ ##
##           	                   mod.3.INSEE                              ##
## ------------------------------------------------------------------------ ##

## ---------------- Male population

y <- D_h
x1 <- rep(Age,length(Year))
x2 <- rep(Year,each= length(Age))
x3 <- ETR_h
x4 <- INSEE_m_h[, as.character(Year)]

## --- Compute the fit

## - Get the span and number of observations participating to the smoothing

nber.p <- get.enp.target(14); nber.p
span <- nber.p / (length(Age)*length(Year)); span

## - Fit the model

mod.3.INSEE_h <- locfit(c(y) ~ lp(c(x1),c(x2),deg = 1, nn = span,  scale=1),ev=dat(), family = fam, link = lk, kern = k, weights=c(x3*x4))

fit.m3.INSEE_h <- matrix(predict(mod.3.INSEE_h),length(Age),length(Year)) * x4
colnames(fit.m3.INSEE_h) <- as.character(Year)

x11();
surface_plot(log(fit.m3.INSEE_h), expression(paste("log ",widehat(mu)[h])), c(min(Age),max(Age), min(Year), max(Year)))

## ------------------------------------------------------------------------ ##
##           	                   mod.3.TG05                               ##
## ------------------------------------------------------------------------ ##

## ---------------- Male population

y <- D_h
x1 <- rep(Age,length(Year))
x2 <- rep(Year,each= length(Age))
x3 <- ETR_h
x4 <- TG05_m_h[, as.character(Year)]

## --- Compute the fit

## - Get the span and number of observations participating to the smoothing

nber.p <- get.enp.target(14); nber.p
span <- nber.p / (length(Age)*length(Year)); span

## - Fit the model

mod.3.TG05_h <- locfit(c(y) ~ lp(c(x1),c(x2),deg = 1, nn = span,  scale=1),ev=dat(), family = fam, link = lk, kern = k, weights=c(x3*x4))

fit.m3.TG05_h <- matrix(predict(mod.3.TG05_h),length(Age),length(Year)) * x4
colnames(fit.m3.TG05_h) <- as.character(Year)

x11();
surface_plot(log(fit.m3.TG05_h), expression(paste("log ",widehat(mu)[h])), c(min(Age),max(Age), min(Year), max(Year)))

## ------------------------------------------------------------------------ ##
##           	                   mod.4.INSEE                              ##
## ------------------------------------------------------------------------ ##

## ---------------- Male population

y <- D_h
x1 <- rep(Age,length(Year))
x2 <- rep(Year,each= length(Age))
x3 <- ETR_h
x4 <- INSEE_m_h[, as.character(Year)]

## --- Compute the fit

## - lf() doesn't work... Use lo()... (Same thing, but slightly less flexible)

## - Fit the model

mod.4.INSEE_h <- gam(c(y) ~ lo(c(x1),c(x2) ,degree = 1, span = 0.3) + lo(c(log(x4)), degree = 1, span = 0.3), family = fam, link = lk, offset = c(log(x3)))

fit.m4.INSEE_h <- exp(matrix(predict(mod.4.INSEE_h),length(Age),length(Year)))
colnames(fit.m4.INSEE_h) <- as.character(Year)

x11();
surface_plot(log(fit.m4.INSEE_h), expression(paste("log ",widehat(mu)[h])), c(min(Age),max(Age), min(Year), max(Year)))

## ------------------------------------------------------------------------ ##
##           	                   mod.4.TG05                                   ##
## ------------------------------------------------------------------------ ##

## ---------------- Male population

y <- D_h
x1 <- rep(Age,length(Year))
x2 <- rep(Year,each= length(Age))
x3 <- ETR_h
x4 <- TG05_m_h[, as.character(Year)]

## --- Compute the fit

## - Fit the model

mod.4.TG05_h <- gam(c(y) ~ lo(c(x1),c(x2) ,degree = 1, span = 0.3) + lo(c(log(x4)), degree = 1, span = 0.3), family = fam, link = lk, offset = c(log(x3)))

fit.m4.TG05_h <- exp(matrix(predict(mod.4.TG05_h),length(Age),length(Year)))
colnames(fit.m4.TG05_h) <- as.character(Year)

x11();
surface_plot(log(fit.m4.TG05_h), expression(log(widehat(mu)[h])), c(min(Age),max(Age), min(Year), max(Year)))

## ------------------------------------------------------------------------ ##
##           	               Save the models                              ##
## ------------------------------------------------------------------------ ##

mod.lk_h <- vector("list",7)
mod.lk_h[[1]] <- mod.1_h
mod.lk_h[[2]] <- mod.2.INSEE_h
mod.lk_h[[3]] <- mod.2.TG05_h
mod.lk_h[[4]] <- mod.3.INSEE_h
mod.lk_h[[5]] <- mod.3.TG05_h
mod.lk_h[[6]] <- mod.4.INSEE_h
mod.lk_h[[7]] <- mod.4.TG05_h

names(mod.lk_h) <- c("M1", "M2.INSEE", "M2.TG05","M3.INSEE", "M3.TG05","M4.INSEE", "M4.TG05")

save(mod.lk_h, file="mod.lk_h.RData")

## ------------------------------------------------------------------------ ##
##           	                 Save the fits                              ##
## ------------------------------------------------------------------------ ##

fit.lk_h <- vector("list",7)
fit.lk_h[[1]] <- fit.m1_h
fit.lk_h[[2]] <- fit.m2.INSEE_h
fit.lk_h[[3]] <- fit.m2.TG05_h
fit.lk_h[[4]] <- fit.m3.INSEE_h
fit.lk_h[[5]] <- fit.m3.TG05_h
fit.lk_h[[6]] <- fit.m4.INSEE_h
fit.lk_h[[7]] <- fit.m4.TG05_h

names(fit.lk_h) <- c("M1", "M2.INSEE", "M2.TG05","M3.INSEE", "M3.TG05","M4.INSEE", "M4.TG05")

save(fit.lk_h, file="fit.lk_h.RData")
