

## ------------------------------------------------------------------------ ##
##  R scripts for " A New Parametric Approach to Smoothing and Forecasting  ##
##                 of Mortality "                                           ##
## ------------------------------------------------------------------------ ##
##  Script        Scripts to forecast with PCA  -  Script n•06.1            ##
## ------------------------------------------------------------------------ ##
##  Description   Script to forecast mortality tables smoothed with robust  ##
##                functional PCA.                                           ##
## ------------------------------------------------------------------------ ##
##  Author        Tomas Julien                                              ##
## ------------------------------------------------------------------------ ##
##  Version       01 - 19/10/2012                                           ##
## ------------------------------------------------------------------------ ##

## ---------------- If necessary,

rm(list=ls(all=TRUE))

## ------------------------------------------------------------------------ ##
##           	       Change the workspace repository                      ##
## ------------------------------------------------------------------------ ##

setwd("...")

## ------------------------------------------------------------------------ ##
##           	               Load functions                               ##
## ------------------------------------------------------------------------ ##

## --- Miscellaneous functions

source("Misc_functions.R")

## --- Graphic functions

require(lattice); require(grid); require(latticeExtra)

## --- Load X11 options

X11.options(type="nbcairo",family="FreeSans")

## --- Demography package

require(demography)

## ------------------------------------------------------------------------ ##
##           	                  Load Data                                 ##
## ------------------------------------------------------------------------ ##

load("D_h.RData")

load("ETR_h.RData")

Age <- 30:90; Years <- 1996:2007
period_forecast <- 2008:2035

## ------------------------------------------------------------------------ ##
##           	                 Load tables                                ##
## ------------------------------------------------------------------------ ##

## --- Local likelihood

load("fit.lk_h.RData")

## --- Create demogdata object

mod.names <- c("M1", "M2.INSEE", "M2.TG05", "M3.INSEE", "M3.TG05", "M4.INSEE", "M4.TG05")

demodata_lk <- vector("list",7)
for (i in 1:7){
	demodata_lk[[i]] <- demogdata(data = fit.lk_h[[i]], pop = D_h, ages = Age, years = Years, type = "mortality", label = mod.names[i], name= "male")
}

## ------------------------------------------------------------------------ ##
##           Decompose fitted curves via a basis function expansion         ##
## ------------------------------------------------------------------------ ##

k.order <- 3

fd_lk <- vector("list",7) 
for (i in 1:7){
	fd_lk[[i]] <- fdm(data = demodata_lk[[i]], serie = "male", max.age=max(Age), method = "classical", order = k.order, lambda = 1)
}

## ------------------------------------------------------------------------ ##
##          Fit univariate time series models to each of the beta_k         ##
## ------------------------------------------------------------------------ ##

forecast_lk <- vector("list",7) 
for (i in 1:7){
	forecast_lk[[i]] <- forecast(fd_lk[[i]], h = length(period_forecast), method="arima", adjust=FALSE, level=95) }

for(i in 1:7){
x11();
print(plot(forecast_lk[[i]], "c", ylab1="B", ylab2="C", mean.lab=expression(mu[x]), main.title="", interaction.title=""))}

fitted_lk_uni <- vector("list",7)
for (i in 1:7){
	fitted_lk_uni[[i]] <- cbind(exp(forecast_lk[[i]]$fitted$y), forecast_lk[[i]]$rate$male)
}

for (i in 1:7){
	x11()
	print(surface_plot(fitted_lk_uni[[i]], expression(varphi[x](t)), c(min(Age),max(Age),min(Year), max(Year)))) }
	
	for (i in 1:7){
	x11()
	print(surface_plot(log(fitted_lk_uni[[i]]), expression(varphi[x](t)), c(min(Age),max(Age),min(Year), max(Year)))) }

## ------------------------------------------------------------------------ ##
##                   Fit ARIMA models to each of the beta_k                 ##
## ------------------------------------------------------------------------ ##

arima.beta <- vector("list", 7)
for (i in 1:7){
	arima.beta[[i]] <- vector("list",k.order)
	for (j in 1:k.order){
		arima.beta[[i]][[j]] <- auto.arima(fd_lk[[i]]$coeff[,j+1], allowdrift=TRUE, start.p=0, start.q=0, ic="bic")
	}
}

## ------------------------------------------------------------------------ ##
##                             Forecast beta_k                              ##
## ------------------------------------------------------------------------ ##

forecast_beta <-  vector("list",7)
for (i in 1:7){
	forecast_beta[[i]] <- vector("list",k.order)
	for (j in 1:k.order){
		forecast_beta[[i]][[j]] <- forecast(arima.beta[[i]][[j]],h=length(period_forecast), level=c(95,99)) } }

## ------------------------------------------------------------------------ ##
##                          Obtain fitted values                            ##
## ------------------------------------------------------------------------ ##

fitted_lk <- vector("list",7)
for(i in 1:7){
	fitted_lk[[i]] <- matrix(,length(Age), length(period_forecast))
	for(j in 1:length(Age)){
		for(k in 1:length(period_forecast)){
			fitted_lk[[i]][j,k] <- as.numeric(fd_lk[[i]]$basis[,1])[j] + fd_lk[[i]]$basis[j,2] * as.numeric(forecast_beta[[i]][[1]]$mean)[k] + fd_lk[[i]]$basis[j,3] * as.numeric(forecast_beta[[i]][[2]]$mean)[k] + fd_lk[[i]]$basis[j,4] * as.numeric(forecast_beta[[i]][[3]]$mean)[k] 
		}
	}
}

## ------------------------------------------------------------------------ ##
##                                 Combine                                  ##
## ------------------------------------------------------------------------ ##

pred.pca_h <- vector("list",7)
for (i in 1:7){
	pred.pca_h[[i]] <- cbind(fit.lk_h[[i]], exp(fitted_lk[[i]]))
	colnames(pred.pca_h[[i]]) <- as.character(c(Year,period_forecast))
}

for (i in 1:7){
	x11()
	print(surface_plot(pred.pca_h[[i]], expression(varphi[x](t)), c(min(Age),max(Age),min(Year), max(Year)))) }
	
for (i in 1:7){
	x11()
	print(surface_plot(log(pred.pca_h[[i]]), expression(varphi[x](t)), c(min(Age),max(Age),min(Year), max(Year)))) }

save(pred.pca_h, file="pred.pca_h.RData")

## ------------------------------------------------------------------------ ##
##                                   Plot                                   ##
## ------------------------------------------------------------------------ ##

col.vec <- c(1,2,2,isfa,isfa,4,4)
pch.vec <- c(NA, 15, 0, 16, 1, 17, 2)
pch.vec <- c(rep(NA,7))
cex.vec <- c(1, 1.2, 1, 1.2, 1, 1.2, 1)
lwd.vec <- 2
lty.vec <- c(1, 1, 3, 1, 3, 1, 3)
leg.name <- c("M1", "M2.INSEE", "M2.TG05", "M3.INSEE", "M3.TG05", "M4.INSEE", "M4.TG05")

## ---------------- Comparisons parameters PCA

x11();
comp.plot.pca1(fd_lk, 1, c(50,138), col.vec, lty.vec, pch.vec, cex.vec, leg.name, Age, c(-8.1,-1), expression(paste("Comparison of ", mu(x))), expression(mu(x)), 5, .7, 1)

x11();
comp.plot.pca1(fd_lk, 2, c(130,138), col.vec, lty.vec, pch.vec, cex.vec, leg.name, Age, c(0.03,.23), expression(paste("Comparison of basis functions: ", phi[1](x))), expression(phi[1](x)), 5, .8, 1)

x11();
comp.plot.pca1(fd_lk, 3, c(130,48), col.vec, lty.vec, pch.vec, cex.vec, leg.name, Age, c(-0.7,.4), expression(paste("Comparison of basis functions: ", phi[2](x))), expression(phi[2](x)), 5, .8, 1)

x11();
comp.plot.pca1(fd_lk, 4, c(130,48), col.vec, lty.vec, pch.vec, cex.vec, leg.name, Age, c(-0.7,.6), expression(paste("Comparison of basis functions: ", phi[3](x))), expression(phi[3](x)), 5, .8, 1)

x11();
comp.plot.pca2(fd_lk, 1, c(130,138), col.vec, lty.vec, pch.vec, cex.vec, leg.name, 1996:2007 , c(-1.3,1.3), expression(paste("Comparison of coefficients: ", beta[t1])), expression(beta[t1]), 1, .9, 1)

x11();
comp.plot.pca2(fd_lk, 2, c(90,148), col.vec, lty.vec, pch.vec, cex.vec, leg.name, 1996:2007 , c(-.05,.07), expression(paste("Comparison of coefficients: ", beta[t2])), expression(beta[t2]), 1, .9, 3)

x11();
comp.plot.pca2(fd_lk, 3, c(90,148), col.vec, lty.vec, pch.vec, cex.vec, leg.name, 1996:2007 , c(-.022,.0225), expression(paste("Comparison of coefficients: ", beta[t3])), expression(beta[t3]), 1, .9, 3)

## ---------------- Comparisons forecast parameters PCA

x11()
projection.plot(fd_lk, forecast_beta, 1, 1996:2007, 2008:2035, c(130,138), col.vec, lty.vec, pch.vec, cex.vec, leg.name, c(-11.3,2), expression(paste("Projection of ", beta[t1],"'s obtained by ARIMA")), 5)

x11()
projection.plot(fd_lk, forecast_beta, 2, 1996:2007, 2008:2035, c(50,40), col.vec, lty.vec, pch.vec, cex.vec, leg.name, c(-.1,.12), expression(paste("Projection of ", beta[t2],"'s obtained by ARIMA")), 5)

x11()
projection.plot(fd_lk, forecast_beta, 3, 1996:2007, 2008:2035, c(130,138), col.vec, lty.vec, pch.vec, cex.vec, leg.name, c(-.022,.0225), expression(paste("Projection of ", beta[t3],"'s obtained by ARIMA")), 5)



